<?php ob_start();

// Check if WooCommerce is active
function check_woocommerce_activation()
{
    if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
        return false;
    }

    return true;
}

// If WooCommerce is not active, stop loading the plugin
if (!check_woocommerce_activation()) {
    return;
}

add_filter('woocommerce_payment_gateways', 'add_woocommerce_gateway');
wp_clear_scheduled_hook('woocommerce_cancel_unpaid_orders');

add_action('plugins_loaded', 'init_woocommerce_gateway_class');

function add_woocommerce_gateway($gateways)
{
    $gateways[] = 'Woocommerce_gateway';
    return $gateways;
}

//disable apppago if total is less than min amount or more than max amount
add_filter('woocommerce_available_payment_gateways', 'disable_apppago_gateway');

function disable_apppago_gateway($available_gateways)
{
    global $woocommerce;

    if (is_checkout()) {
        $order_total = $woocommerce->cart->total;

        if (isset($available_gateways['apppago_gateway'])) {
            if ($order_total < $available_gateways['apppago_gateway']->settings["min_amount"] || $order_total > $available_gateways['apppago_gateway']->settings["max_amount"]) {
                unset($available_gateways['apppago_gateway']);
            }
        }
    }

    return $available_gateways;
}

function custom_gateway_icon($icon, $id)
{
    if ($id == "apppago_gateway") {
        return '<img src="' . plugins_url('apppago_smallcrop.png', __FILE__) . '" style="max-width: 4rem; text-align: center" > ';
    }

}

function init_woocommerce_gateway_class()
{

    class Woocommerce_gateway extends WC_Payment_Gateway
    {

        public function __construct()
        {
            $this->id = 'apppago_gateway';
            $this->description = '';
            $this->title = 'Paga in comode quote con AppPago';
            $this->icon = add_filter('woocommerce_gateway_icon', 'custom_gateway_icon', 10, 2);
            $this->has_fields = true;
            $this->method_title = 'AppPago';
            $this->method_description = 'Paga in comode quote con AppPago';
            $this->supports = array(
                'products'
            );

            $this->init_form_fields();
            $this->init_settings();

            $GLOBALS["api_key"] = $this->settings["api_key"];
            $GLOBALS["test_mode"] = $this->settings["test_mode"];

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                "test_mode" => array(
                    "title" => __("Modalità di Testing", "woocommerce"),
                    "type" => "checkbox",
                    "label" => __("Abilità Modalità di Testing", "woocommerce"),
                    "default" => "no"
                ),
                "merchant_code" => array(
                    "title" => __("Merchant Code", "woocommerce"),
                    "type" => "text",
                    "description" => __("Inserisci il tuo merchant code", "woocommerce"),
                    "default" => ""
                ),
                "api_key" => array(
                    "type" => "hidden",
                    "default" => bin2hex(random_bytes(16))
                ),
                "vat" => array(
                    "title" => __("VAT", "woocommerce"),
                    "type" => "text",
                    "description" => __("Inserisci la tua partita IVA", "woocommerce"),
                    "default" => ""
                ),
                "max_amount" => array(
                    "title" => __("Importo Massimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci l'importo massimo", "woocommerce"),
                    "default" => ""
                ),
                "min_amount" => array(
                    "title" => __("Importo Minimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci l'importo minimo", "woocommerce"),
                    "default" => ""
                ),
                "installments_type" => array(
                    "title" => __("Tipo di mensilità", "woocommerce"),
                    "type" => "select",
                    "description" => __("Scegli il tipo di mensilità da offrire", "woocommerce"),
                    "default" => "none",
                    "options" => array(
                        "none" => "Scegli...",
                        "fixed" => "Fisso",
                        "range" => "Range",
                        "select" => "Selezione Multipla"
                    )
                ),
                "fixed_installments_number" => array(
                    "title" => __("Numero di mensilità", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci il numero di mensilità fisso da offrire", "woocommerce"),
                    "default" => ""
                ),
                "range_installments_min" => array(
                    "title" => __("Numero di mensilità Minimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci il numero minimo di mensilità da offrire", "woocommerce"),
                    "default" => ""
                ),
                "range_installments_max" => array(
                    "title" => __("Numero di mensilità Massimo", "woocommerce"),
                    "type" => "number",
                    "description" => __("Inserisci il numero massimo di mensilità da offrire", "woocommerce"),
                    "default" => ""
                ),
                "select_installments_list" => array(
                    "title" => __("Lista mensilità da Offrire", "woocommerce"),
                    "type" => "multiselect",
                    "description" => __("Scegli le mensilità da offrire", "woocommerce"),
                    "default" => "none",
                    "options" => array(
                        "3" => "3",
                        "4" => "4",
                        "5" => "5",
                        "6" => "6",
                        "7" => "7",
                        "8" => "8",
                        "9" => "9",
                        "10" => "10",
                        "11" => "11",
                        "12" => "12"
                    )
                ),
                "iban_allowed" => array(
                    "title" => __("Permetti IBAN", "woocommerce"),
                    "type" => "checkbox",
                    "label" => __("Abilita l'acquisto tramite IBAN", "woocommerce"),
                    "default" => "no"
                ),
                "company_allowed" => array(
                    "title" => __("Permetti Partita IVA", "woocommerce"),
                    "type" => "checkbox",
                    "label" => __("Abilita l'acquisto tramite partita IVA", "woocommerce"),
                    "default" => "no"
                )
            );

            add_action('admin_head', function () {

                if (!is_admin())
                    return;
                else {
                    echo '
                <script>
                    document.addEventListener("DOMContentLoaded", () => {
                        let installmentsType = document.getElementById("woocommerce_apppago_gateway_installments_type");
                        console.log("DOM LOADED");
                        let fixedInstallmentsNumber = document.getElementById(
                            "woocommerce_apppago_gateway_fixed_installments_number"
                        ).parentNode.parentNode.parentNode;

                        let rangeInstallmentsMin = document.getElementById(
                            "woocommerce_apppago_gateway_range_installments_min"
                        ).parentNode.parentNode.parentNode;
                        let rangeInstallmentsMax = document.getElementById(
                            "woocommerce_apppago_gateway_range_installments_max"
                        ).parentNode.parentNode.parentNode;

                        let selectInstallmentsList = document.getElementById(
                            "woocommerce_apppago_gateway_select_installments_list"
                        ).parentNode.parentNode.parentNode;

                        if (installmentsType.value == "fixed") {
                            fixedInstallmentsNumber.style.display = "";
                            rangeInstallmentsMin.style.display = "none";
                            rangeInstallmentsMax.style.display = "none";
                            selectInstallmentsList.style.display = "none";
                        } else if (installmentsType.value == "range") {
                            fixedInstallmentsNumber.style.display = "none";
                            selectInstallmentsList.style.display = "none";
                            rangeInstallmentsMin.style.display = "";
                            rangeInstallmentsMax.style.display = "";
                        } else if (installmentsType.value == "select") {
                            fixedInstallmentsNumber.style.display = "none";
                            rangeInstallmentsMin.style.display = "none";
                            rangeInstallmentsMax.style.display = "none";
                            selectInstallmentsList.style.display = "";
                        } else {
                            fixedInstallmentsNumber.style.display = "none";
                            rangeInstallmentsMin.style.display = "none";
                            rangeInstallmentsMax.style.display = "none";
                            selectInstallmentsList.style.display = "none";
                        }

                        installmentsType.addEventListener("change", (e) => {
                            if (e.target.value == "fixed") {
                                fixedInstallmentsNumber.style.display = "";
                                rangeInstallmentsMin.style.display = "none";
                                rangeInstallmentsMax.style.display = "none";
                                selectInstallmentsList.style.display = "none";
                            } else if (e.target.value == "range") {
                                fixedInstallmentsNumber.style.display = "none";
                                rangeInstallmentsMin.style.display = "";
                                rangeInstallmentsMax.style.display = "";
                                selectInstallmentsList.style.display = "none";
                            } else if (e.target.value == "select") {
                                fixedInstallmentsNumber.style.display = "none";
                                rangeInstallmentsMin.style.display = "none";
                                rangeInstallmentsMax.style.display = "none";
                                selectInstallmentsList.style.display = "";
                            } else {
                                fixedInstallmentsNumber.style.display = "none";
                                rangeInstallmentsMin.style.display = "none";
                                rangeInstallmentsMax.style.display = "none";
                                selectInstallmentsList.style.display = "none";
                            }
                        });
                    });

                </script>';
                }
            });
        }

        public function process_admin_options()
        {
            global $base_url_testing, $base_url_production, $current_version;


            $_POST["woocommerce_apppago_gateway_api_key"] = bin2hex(random_bytes(16));
            $merchant_code = isset($_POST["woocommerce_apppago_gateway_merchant_code"]) ? $_POST["woocommerce_apppago_gateway_merchant_code"] : "";
            $api_key = $_POST["woocommerce_apppago_gateway_api_key"];
            $vat = isset($_POST["woocommerce_apppago_gateway_vat"]) ? $_POST["woocommerce_apppago_gateway_vat"] : "";
            $min_amount = isset($_POST["woocommerce_apppago_gateway_min_amount"]) ? $_POST["woocommerce_apppago_gateway_min_amount"] : "";
            $max_amount = isset($_POST["woocommerce_apppago_gateway_max_amount"]) ? $_POST["woocommerce_apppago_gateway_max_amount"] : "";
            $installmentsType = isset($_POST["woocommerce_apppago_gateway_installments_type"]) ? $_POST["woocommerce_apppago_gateway_installments_type"] : "";
            $fixedInstallmentsNumber = isset($_POST["woocommerce_apppago_gateway_fixed_installments_number"]) ? $_POST["woocommerce_apppago_gateway_fixed_installments_number"] : "";
            $rangeInstallmentsMin = isset($_POST["woocommerce_apppago_gateway_range_installments_min"]) ? $_POST["woocommerce_apppago_gateway_range_installments_min"] : "";
            $rangeInstallmentsMax = isset($_POST["woocommerce_apppago_gateway_range_installments_max"]) ? $_POST["woocommerce_apppago_gateway_range_installments_max"] : "";
            $select_installments_list = isset($_POST["woocommerce_apppago_gateway_select_installments_list"]) ? $_POST["woocommerce_apppago_gateway_select_installments_list"] : [];
            $ibanAllowed = isset($_POST["woocommerce_apppago_gateway_iban_allowed"]) ? $_POST["woocommerce_apppago_gateway_iban_allowed"] : "0";
            $companyAllowed = isset($_POST["woocommerce_apppago_gateway_company_allowed"]) ? $_POST["woocommerce_apppago_gateway_company_allowed"] : "0";

            $args = array(
                "body" => array(
                    "platform" => "woocommerce",
                    "version" => $current_version,
                    "host" => get_site_url(),
                    "merchant_code" => $merchant_code,
                    "api_key" => $api_key,
                    "vat" => $vat,
                    "max_amount" => $max_amount,
                    "min_amount" => $min_amount,
                    "installments_type" => $installmentsType,
                    "fixed_installments_number" => $fixedInstallmentsNumber,
                    "range_installments_min" => $rangeInstallmentsMin,
                    "range_installments_max" => $rangeInstallmentsMax,
                    "select_installments_list" => $select_installments_list,
                    "iban_allowed" => $ibanAllowed,
                    "company_allowed" => $companyAllowed
                )
            );

            // $base_url = isset($_POST["woocommerce_apppago_gateway_test_mode"]) && $_POST["woocommerce_apppago_gateway_test_mode"] == 1 ? $base_url_testing : $base_url_production;

            $base_url = $base_url_production;

            $response = wp_remote_post($base_url . "api/config", $args);
            $response_body = wp_remote_retrieve_body($response);

            if (wp_remote_retrieve_response_code($response) == 500) {
                WC_Admin_Settings::add_error("Errore del server. riprovare più tardi");
                return false;
            } else if (wp_remote_retrieve_response_code($response) == 400) {
                WC_Admin_Settings::add_error("Errore di validazione del merchant Id. Controllare i dati inseriti");
                return false;
            } else if (wp_remote_retrieve_response_code($response) == 200) {
                parent::process_admin_options();
                return true;
            } else {
                WC_Admin_Settings::add_error("Errore sconosciuto. Controllare i dati inseriti");
                return false;
            }
        }

        public function payment_fields()
        {

            //SHOW A DESCRIPTION
            if ($this->description) {
                echo wpautop(wp_kses_post("$this->description"));
            }

            // Add this action hook if you want your custom payment gateway to support it
            do_action('woocommerce_credit_card_form_start', $this->id);


            echo '<div class="form-row form-row-wide">
                <input type="hidden" name="clientType" value="privato">
                <label>Codice Fiscale:</label>
                <input id="taxCode" class="input-text wc-credit-card-form-card-number" type="text" autocomplete="off" placeholder="Inserisci il tuo codice fiscale" name="taxCode" />
                </div>
                <div class="clear"></div>';



            do_action('woocommerce_credit_card_form_end', $this->id);
            echo '<div class="clear"></div></fieldset>';

        }

        public function payment_scripts()
        {
        }



        public function process_payment($order_id)
        {
            global $base_url_testing, $base_url_production, $woocommerce;
            $base_url = $this->settings["test_mode"] == "yes" ? $base_url_testing : $base_url_production;

            $order = new WC_Order($order_id);
            $order->update_status('failed', __('L\'utente è nel flow di checkout', 'woocommerce'));

            if (!isset($_POST["clientType"])) {
                wc_add_notice("Selezionare il tipo di cliente", "error");
                return array(
                    'result' => 'fail'
                );
            }
            if ($_POST["clientType"] == "privato" && !preg_match("/^[A-Z]{6}[0-9]{2}[A-Z][0-9]{2}[A-Z][0-9]{3}[A-Z]$/", strtoupper($_POST["taxCode"]))) {
                wc_add_notice("Il codice fiscale inserito non è valido", "error");
                return array(
                    'result' => 'fail'
                );

            } else if ($_POST["clientType"] == "azienda" && !preg_match("/^[0-9]{11}$/", strtoupper($_POST["taxCode"]))) {
                wc_add_notice("La partita IVA inserita non è valida", "error");
                return array(
                    'result' => 'fail'
                );
            }

            $olMerchantTrxId = substr(str_shuffle("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"), 0, 5);

            try {
                $body = array(
                    "olMerchantTrxId" => $olMerchantTrxId,
                    "merchantId" => $this->settings["merchant_code"],
                    "loanTotalAmount" => $order->get_total(),
                    "installmentNumber" => $this->settings["installments_type"] == "fixed" ? $this->settings["fixed_installments_number"] : "",
                    "firstname" => $order->get_billing_first_name(),
                    "lastname" => $order->get_billing_last_name(),
                    "taxCode" => strtoupper($_POST["taxCode"]),
                    "residenceAddress" => $order->get_billing_address_1(),
                    "residenceCity" => $order->get_billing_city(),
                    "residenceState" => $order->get_billing_state(),
                    "residencePostalCode" => $order->get_billing_postcode(),
                    "email" => $order->get_billing_email(),
                    "phone" => $order->get_billing_phone(),
                    "merchantRedirect" => get_site_url() . "/wp-json/apppago/callback/",
                    "merchant_order_id" => $order_id,
                );

                $url = wp_remote_post(
                    $base_url . "api/order",
                    array(
                        'body' => $body
                    )
                );
                $url = wp_remote_retrieve_body($url);

                $url = json_decode($url, true);


                if ($url["redirectUrl"]) {
                    $order->update_status('on-hold', __('Flusso AppPago: in attesa che l\'utente completi il flusso', 'woocommerce'));
                    return array(
                        'result' => 'success',
                        'redirect' => $url["redirectUrl"]
                    );
                } else {
                    $order->update_status('failed', __('Ordine annullato, impossibile contattare server AppPago', 'woocommerce'));
                    wc_add_notice("Errore sconosciuto. Riprovare più tardi", "error");
                    return array(
                        'result' => 'fail'
                    );
                }
            } catch (Exception $e) {
                $order->update_status('failed', __('Ordine annullato, impossibile contattare server AppPago', 'woocommerce'));
                wc_add_notice("Errore sconosciuto. Riprovare più tardi", "error");
                return;
            }
        }
    }
}